###############################################################################
# (c) 2005-2020 Copyright, Real-Time Innovations.  All rights reserved.       #
# No duplications, whole or partial, manual or electronic, may be made        #
# without express written permission.  Any such copies, or revisions thereof, #
# must display this notice unaltered.                                         #
# This code contains trade secrets of Real-Time Innovations, Inc.             #
###############################################################################

from time import sleep

# Updating the system path is not required if you have pip-installed
# rticonnextdds-connector
from sys import path as sys_path
from os import path as os_path
file_path = os_path.dirname(os_path.realpath(__file__))
sys_path.append(file_path + "/../../../")

import rticonnextdds_connector as rti
import ctypes

ARRAY_LEN = 1024
# Expose the DynamicData API which we are going to use
# It has the following prototype:
# DDS_ReturnCode_t DDS_DynamicData_set_octet_array(
#       DDS_DynamicData *,
#       const char *,
#       DDS_DynamicDataMemberId,
#       DDS_UnisgnedLong
#       const DDS_Octet *)
# On Windows we would need to explicitly load the nddsc.dll
# nddsc_library = cdll.LoadLibrary("C:\\Path\\To\\nddsc.dll")
# And then use nddsc_library variable above instead of
# rti.connector_binding.library
DDS_DynamicData_set_octet_array = rti.connector_binding.library.DDS_DynamicData_set_octet_array
DDS_DynamicData_set_octet_array.restype = ctypes.c_int
DDS_DynamicData_set_octet_array.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_long, ctypes.c_ulong, ctypes.c_void_p]

def setSequenceNatively(output, field_name, python_arr):
    # We need access to the native DynamicData sample which we can supply
    # to the DDS C APIs
    native_dynamic_data = output.instance.native

    # We then need to build the sequence such that it is compatible with C.
    # This is accomplished using ctypes
    # Unsigned long for the length of the array
    array_length = ctypes.c_ulong(len(python_arr))
    # Create a new type
    in_array_type = ctypes.c_char * len(python_arr)
    # Set the member ID to 0 (translates to unknown in DDS C DynamicData API)
    member_id = 0
    # Create an instance of our new type using the python array
    in_array = in_array_type(*python_arr)
    # Here we set the values into the member
    DDS_DynamicData_set_octet_array(
            ctypes.cast(native_dynamic_data, ctypes.c_void_p),
            field_name.encode("utf8"),
            member_id,
            array_length,
            ctypes.byref(in_array))

with rti.open_connector(
        config_name="MyParticipantLibrary::MyPubParticipant",
        url=file_path + "/../Example.xml") as connector:

    output = connector.get_output("MyPublisher::MyImageWriter")
    output.wait_for_subscriptions()

    r_data = [b'f'] * ARRAY_LEN
    setSequenceNatively(output, "r_data", r_data)
    g_data = [b'4'] * ARRAY_LEN
    setSequenceNatively(output, "g_data", g_data)
    b_data = [b'0'] * ARRAY_LEN
    setSequenceNatively(output, "b_data", b_data)

    print("Writing...")
    for i in range(1, 100):
        output.write()
        sleep(0.5)
    print("Exiting...")
    output.wait()